
/* Name: main.c
 * Project: USB music leds
 * Author: alexs alexs@alexsp.ru
 * Creation Date: 2010-11-02
 * Tabsize: 4
 * Copyright: (c) 2010 by AlexS 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

	This program use Objective Development's firmware-only USB driver
	for Atmel AVR microcontrollers. For more information please visit
	http://www.obdev.at/vusb/ and please read: usbdrv/README.txt 
	usbdrv/License.txt usbdrv/CommercialLicense.txt

*/


#if defined (__AVR_ATmega168__) || defined (__AVR_ATmega168A__) || (__AVR_ATmega88__) || defined (__AVR_ATmega88A__) || (__AVR_ATmega48__) || defined (__AVR_ATmega48A__)
#define __Timer0_CTC__ 1
#elif defined (__AVR_ATmega8__)
#undef __Timer0_CTC__
#else
#error "The device is not compatible"
#endif


#define ON_VALUE 12 //      X/16
#define INIT_SPEED 40 //   -
#define PWM_F 1000 //   


#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */

#include <avr/pgmspace.h>   /* required by usbdrv.h */
#include "usbdrv/usbdrv.h"

#include "leds.h"

/*  HID  */

static PROGMEM char deviceDescrHID[] = {    /* USB device descriptor */
    18,         /* sizeof(usbDescriptorDevice): length of descriptor in bytes */
    USBDESCR_DEVICE,        /* descriptor type */
    0x01, 0x01,             /* USB version supported */
    0,                      /* device class: composite */
    0,                      /* subclass */
    0,                      /* protocol */
    8,                      /* max packet size */
    0xc0, 0x16,
    0xdf, 0x05,
// 	0xaa, 0xaa,
//	0x22, 0xef,
    USB_CFG_DEVICE_VERSION, /* 2 bytes */
    1,                      /* manufacturer string index */
    2,                      /* product string index */
    0,                      /* serial number string index */
    1,                      /* number of configurations */
};

static PROGMEM char configDescrHID[] = {    /* USB configuration descriptor */
    9,          /* sizeof(usbDescriptorConfiguration): length of descriptor in bytes */
    USBDESCR_CONFIG,/* descriptor type */
    18 + 7 + 9 +7, 0,  /* total length of data returned (including inlined descriptors) */
    1,          /* number of interfaces in this configuration */
    1,          /* index of this configuration */
    0,          /* configuration name string index */
#if USB_CFG_IS_SELF_POWERED
    USBATTR_SELFPOWER,  /* attributes */
#else
    USBATTR_BUSPOWER,   /* attributes */
#endif
    USB_CFG_MAX_BUS_POWER/2,    /* max USB current in 2mA units */
/* interface descriptor follows inline: */

    9,          /* sizeof(usbDescrInterface): length of descriptor in bytes */
    USBDESCR_INTERFACE, /* descriptor type */
    0,          /* index of this interface */
    0,          /* alternate setting for this interface */
    2,   /* endpoints excl 0: number of endpoint descriptors to follow */
    3,          /* interface class: HID */
    0,          /* subclass */
    0,          /* protocol */
    0,          /* string index for interface */

    9,          /* sizeof(usbDescrHID): length of descriptor in bytes */
    USBDESCR_HID,   /* descriptor type: HID */
    0x11, 0x01, /* BCD representation of HID version */
    0x00,       /* target country code */
    0x01,       /* number of HID Report (or other HID class) Descriptor infos to follow */
    0x22,       /* descriptor type: report */
    USB_CFG_HID_REPORT_DESCRIPTOR_LENGTH, 0,  /* total length of report descriptor */

    7,          /* sizeof(usbDescrEndpoint) */
    USBDESCR_ENDPOINT,  /* descriptor type = endpoint */
    0x81,       /* IN endpoint number 1 */
    0x03,       /* attrib: Interrupt endpoint */
    8, 0,       /* maximum packet size */
    USB_CFG_INTR_POLL_INTERVAL, /* in ms */

    7,          /* sizeof(usbDescrEndpoint) */
    USBDESCR_ENDPOINT,  /* descriptor type = endpoint */
    0x02,       /* IN endpoint number 1 */
    0x03,       /* attrib: Interrupt endpoint */
    8, 0,       /* maximum packet size */
    USB_CFG_INTR_POLL_INTERVAL, /* in ms */
};

// HID Descriptor    (Radan)
//       usb.org
//  ,    
PROGMEM char usbDescriptorHidReport[20] = {
0x06 , 0x00 , 0xFF,
0x09 , 0x01,
0xA1 , 0x02,
0x09 , 0x01,
0x15 , 0x00,
0x25 , 0xFF,
0x75 , 0x08,
0x95 , 0x01,
0x91 , 0x02,
0xC0
};

uchar usbFunctionDescriptor(usbRequest_t *rq)
{
uchar *p, len = 0;

        if(rq->wValue.bytes[1] == USBDESCR_DEVICE){
            p = (uchar *)deviceDescrHID;
            len = sizeof(deviceDescrHID);
        }else if(rq->wValue.bytes[1] == USBDESCR_CONFIG){
            p = (uchar *)configDescrHID;
            len = sizeof(configDescrHID);
        }else{  /* must be HID descriptor */
            p = (uchar *)(configDescrHID + 18);
            len = 9;
        }

    usbMsgPtr = p;
    return len;
}

uchar usbFunctionSetup(uchar data[8])
{
    return 0; 	//   usb_control_msg -  ... 
				//     
}

/* --------- END OF  HID   */


typedef struct LED_t {
	uint8_t * const 	port; //  
	const uint8_t 		bit; // "" 
	volatile uint16_t	pwm; //   
	volatile uint8_t	led_st; //  LED 0-off 1-on

	const uint8_t		inmask; //    0x00, 0x40  0x80
	const uint8_t		inbit; //    ;
	} Led;

void led_on(const Led *);
void led_off(const Led *);
void led_pwm();
void led_val();
void change_led(const uint8_t);

volatile uint8_t pwm_f=0; //   Timer1  
volatile uint8_t onoff_f=0; //   Timer0   -

volatile uint16_t  on_value; //    
uint16_t pwm_m=1; //   

Led Leds[18]=LEDS_ARRAY; // included in leds.h   

#ifndef __Timer0_CTC__
volatile uint8_t t0ctc; //     -  mega8 ..
						// Timer0    CTC
#endif

void usbFunctionWriteOut(uchar *data, uchar len) //    usb_interrupt
{
uint8_t cmd,cmd_f,cmd_v;

	cmd_f=data[0]&0xc0; // 2-     00-10:-  11- 

    if(usbRxToken != 0){
    	if (cmd_f!=0xc0) {
			change_led(data[0]);
			}else{
			// 
				cmd=data[0]&0x30; //   
				cmd_v=data[0]&0x0f; // 
				if (cmd==0x00) {
					on_value=(uint16_t)(0xffff>>(16-cmd_v));
					} //    
				if (cmd==0x10) { //    
#if defined __Timer0_CTC__
					OCR0A&=0xf0;
					OCR0A|=cmd_v;
#else
					t0ctc&=0xf0;
					t0ctc|=cmd_v;
#endif
				}

				if (cmd==0x20) { //    
#if defined __Timer0_CTC__
					OCR0A&=0x0f;
					OCR0A|=(cmd_v<<4);
#else
					t0ctc&=0x0f;
					t0ctc|=(cmd_v<<4);
#endif
				}
			}
    }
//	usbFunctionInterrupt();
}
/* ------------------------------------------------------------------------- */
void hw_init();

int main(void)
{

	wdt_enable(WDTO_1S);
	hw_init();
    usbInit();
    usbDeviceDisconnect();  /* enforce re-enumeration, do this while interrupts are disabled! */
    _delay_ms(100);
    usbDeviceConnect();
    sei();

    for(;;){                /* main event loop */
        wdt_reset();
        usbPoll();

		if (pwm_f) { //  -
			pwm_f--;
			led_pwm();
		}

		if (onoff_f) { //  -
			onoff_f--;
			led_val();
		}
    }
return 0;
}


/* ------------------------------------------------------------------------- */

// PWM - timer1
ISR(TIMER1_COMPA_vect) {
pwm_f=1;
}


// On / OFF Timer0
#if defined __Timer0_CTC__
ISR(TIMER0_COMPA_vect){
#else
ISR(TIMER0_OVF_vect) {
TCNT0=0xff-t0ctc;
#endif

onoff_f=1;
}


void led_on(const Led *led) { *led->port|=led->bit;}
void led_off(const Led *led) { *led->port&=~(led->bit);}


//  PWM   
void led_pwm() {
	uint8_t i;
	for (i=0;i<18;i++) {
		if (Leds[i].pwm&pwm_m) 
			{
			led_on(&Leds[i]);
		}else{
			led_off(&Leds[i]);
		}
	}
	pwm_m=(pwm_m<<1);
	if (!pwm_m) pwm_m=1;
}

//          
void led_val() {
	uint8_t i;
	for (i=0;i<18;i++) {
		if (Leds[i].led_st) {
			Leds[i].pwm=(Leds[i].pwm<<1)+1;
		}else{
			Leds[i].pwm=(Leds[i].pwm>>1);
		}
	}
}


/*
 ""   -       
led_st        
*/
void change_led(const uint8_t in_byte) {
	uint8_t i,m,b;

	m=in_byte&0xC0; //   6  7
	b=in_byte&0x3f; //   
	for (i=0;i<18;i++) {
		if (Leds[i].inmask==m){
			if (Leds[i].inbit&b) {
				Leds[i].led_st=1;
				Leds[i].pwm|=on_value;
			}else{
				Leds[i].led_st=0;
			}
		}
	}
}

void hw_init() { //   

/* Init on-off Timer (8 bit) */
#if defined __Timer0_CTC__
	OCR0A=INIT_SPEED;
	TIMSK0=(1<<OCIE0A);
	TCCR0A=(1<<WGM01);
	TCCR0B=(1<<CS02)|(1<<CS00); // Clk/1024
#else
	t0ctc=INIT_SPEED;
	TCNT0=0xff-t0ctc; //  CTC
	TIMSK=(1<<TOIE0);
	TCCR0=(1<<CS02)|(1<<CS00); // Clk/1024
#endif

	
/* Init PWM (16 bit) */
	OCR1A =(F_CPU/PWM_F)/16;
#ifdef __Timer0_CTC__
	TIMSK1=(1<<OCIE1A);
#else
	TIMSK|=(1<<OCIE1A);
#endif

	TCCR1A=0;
	TCCR1B=(1<<WGM12)|(1<<CS10);
#ifdef __Timer0_CTC__
	TCCR1C=0;
#endif

// Init ports for output 0
	PORTB=0;
	PORTC=0;
	PORTD=0;
	DDRB=0x3f;
	DDRC=0x3f;
	DDRD=0xeb;
	on_value=(uint16_t)(0xffff>>(16-ON_VALUE)); //   =1  
}
